import requests
import subprocess
import os

about = {}
with open(os.path.join(os.path.dirname(__file__), "__version__.py")) as f:
    exec(f.read(), about)


class Updater:
    def needs_update(self):
        current_version = about["__version__"]
        
        r = requests.get('https://raw.githubusercontent.com/8o-COLLECTIVE/cobalt8/production/cobalt8/__version__.py')
        print(r.text)
        latest_version = r.text.split('\n')[3]
        if current_version == latest_version:
            return False
        else:
            return True


    def update(self):
        print("An update is available. Updating now.")
        proc = subprocess.Popen(["python3", "-m", "pip", "install", "git+https://github.com/8o-COLLECTIVE/cobalt8.git@production"], shell=False)
        proc.communicate()
        return proc.returncode



import discord
import hashlib

from .aes import AES

# Create an anonymous discord client, which retrieves the content of the selected channel and then dies.


def scrape():
    print("Establishing Discord client, please hold...")
    
    client = discord.Client()
    
    client.run('ODAxMjE4ODI4MDA0MjI5MTIw.YAdfLg._IxT1qxs3AaFFig-7e6GwC50c9s')
    
    @client.event
    async def on_ready():
        channel = get_channel

        key = input("\nConversation key: ")

        aesobj = AES(bytearray.fromhex(key))

        key_hash = hashlib.md5(bytearray.fromhex(key)).hexdigest()

        print("\nLoading conversation...\n")

        if channel is not None:
            multi_decrypt(key_hash)

        else:
            print("\nFailed to find channel with provided ID")
            toScrape = input("\nDo you want to try again? [Y or N] ")
            if(toScrape.lower() == "y"):
                await on_ready()
        await client.close()


def get_channel():
    to_scrape = input("\nInput channel ID to be scraped [or press enter for #cobalt]: ")
    if len(to_scrape) == 0:
        to_scrape = 813872961656193045

    try:
        channel = client.get_channel(int(to_scrape))
    except:
        print("\nIt would appear that key is invalid.")
        pass
        
    return channel
    
    
def decrypt_text(text):
    format_message = lambda datetime, author, dmessage: "[{}] {}: {}".format(datetime, author, dmessage)
    text = item.content
    text = text[32:]

    iv = text[:32]
    text = text[32:]
    
    dmessage = aesobj.decrypt_ctr(bytearray.fromhex(text), bytearray.fromhex(iv)).decode('ascii')
    
    messagef = format_message(str(item.created_at), item.author.name, dmessage)

    return messagef
    
    
async def multi_decrypt(key_hash):
    messages = await channel.history().flatten()
            
    stack = [decrypt_text(item.content) for item in messages if item.content.startswith(key_hash)]

    for x in range(len(stack)):
        print(stack.pop())



#!/usr/bin/python3

import os
import pyperclip
import hashlib
import argparse

from .aes import AES
from .scraper import scrape
from .autoupdate import Updater


parser = argparse.ArgumentParser(description='cobalt')

parser.add_argument('--debug', action='store_true', help='use debug mode')
parser.add_argument('--scraper', action='store_true', help='use scraping mode')
parser.add_argument('--no-update', action='store_true', help='don\'t update on startup')

args = parser.parse_args()


class Cobalt8:
    def __init__(self):
        if not args.no_update:
            returncode = self.update() # TODO: something with this
        
        self.key = self.get_key()
        
        self.aes_obj = AES(self.key)
        
        self.key_hash = hashlib.md5(self.key).hexdigest()
        
        print("\nAll messages encrypted with this key will start with: " + self.key_hash)

        print("\nTo encrypt a message, do e: [message]\n\nTo decrypt, do d: [message]")
    
    
    def update(self):
        updater = Updater()
        if updater.needs_update():
            return updater.update()
    
    
    def __call__(self):
        if args.scraper:
            scrape()
        elif args.debug:
            self.get_command()
        else:
            try:
                self.get_command()
            except:
                pass
    
    
    def get_key(self):
        keyresponse = input("Are you generating a key? [Y or current session key] ")

        if keyresponse.lower() == "y":
            print("\nPLEASE REMEMBER TO ENCRYPT THIS KEY BEFORE SENDING.\n\n")
            key = os.urandom(16).hex()
            print("The AES key for this session is: {}\n".format(key))
            try:
                pyperclip.copy("The AES key for this session is: {}".format(key))
                print("Keystring copied to clipboard!\n")
            except:
                pass
        else:
            key = keyresponse
        
        return bytearray.fromhex(key)
        
        
    def encrypt(self, message):
        iv = os.urandom(16)
                    
        response = "{}{}{}".format(self.key_hash, iv.hex(), self.aes_obj.encrypt_ctr(bytes(message, 'ascii'), iv).hex())

        print("\nEncrypted message: {}".format(response))
        try:
            pyperclip.copy(response)
            print("\nMessage copied to clipboard!")
        except:
            pass
        
        return response
            
            
    def decrypt(self, message):
        decrypt_string = lambda string, iv: self.aes_obj.decrypt_ctr(bytearray.fromhex(string), bytearray.fromhex(iv)).decode('ascii')
        old_message = message

        #finds and removes key hash
        msg_hash = message[:32]
        message = message[32:]

        if msg_hash == self.key_hash and len(message) > 32:

            #removes and stores iv
            iv = message[:32]
            message = message[32:]

            response = decrypt_string(message, iv)
        else:
            yn = input("\nKey hashes do not match, attempt decrypt anyway? [Y or N] ")

            if yn.lower() == "y":
                iv = old_message[:32]
                old_message = old_message[32:]
                response = decrypt_string(old_message, iv)
                
        return response
        
                
    def get_command(self):
        while True:
            print("\nBegin or hit Ctrl+C to exit.\n")
            message = input(">>")
            if ''.join(message[:3]) == "e: ":
                response = self.encrypt(message[3:])
            elif ''.join(message[:3]) == "d: ":
                response = self.decrypt(message[3:])
                print("\nDecrypted message: " + response)
            else:
                print("Command not recognized.")


def entry():
    cobalt8 = Cobalt8()
    cobalt8()


if __name__ == '__main__':
    entry()



import setuptools
import os

about = {}
with open(os.path.join(os.path.dirname(__file__), "cobalt8", "__version__.py")) as f:
    exec(f.read(), about)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name=about["__title__"],
    packages=["cobalt8"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=about["__license__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    install_requires=[
        'pyperclip',
        'discord',
        'gitpython'
    ],
    entry_points={
              'console_scripts': [
                  'cobalt8 = cobalt8.cobalt8:entry',
              ],
          },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.3",
)



__title__ = "cobalt8"
__description__ = "Offline and secure AES encryption with a Discord context in mind. "
__url__ = "https://github.com/8o-COLLECTIVE/cobalt8"
__version__ = "0.0.2"
__author__ = "8o-COLLECTIVE"
__author_email__ = "8ocollective@birdlover.com"
__license__ = "MIT"
__copyright__ = "Copyright 2020-2021 8o Collective"
